package net.wanji.utc.service.control;

import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.po.SaveToUtcPO;
import net.wanji.databus.vo.*;
import java.util.Date;

/**
 * @author duanruiming
 * @date 2023/01/12 9:44
 */
public interface ControlCommandStrategyService {
    /**
     * 方案下发
     */
    JsonViewObject schemeSendStrategy(SchemeSendVO schemeSendVO) throws Exception;

    /**
     * 计划下发
     *
     * @param planSendVO 计划下发VO
     * @return
     */
    JsonViewObject planSendStrategy(PlanSendVO planSendVO) throws Exception;

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return
     */
    JsonViewObject scheduleSendStrategy(ScheduleSendVO scheduleSendVO) throws Exception;

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    JsonViewObject lockControlStrategy(ControlCommandVO commandVO) throws Exception;

    /**
     * 步进控制
     *
     * @param code    信号机编号
     * @param command 1 开始步进 0 取消步进
     * @param stepNum 0 顺序步进 n 跳过n个相位
     * @return
     * @throws Exception 异常
     */
    JsonViewObject stepControlStrategy(String code, Integer command, Integer stepNum) throws Exception;

    /**
     * 设置信号机控制模式
     *
     * @param code        信号机编号
     * @param command     指令 1 开 0 关
     * @param commandType 命令类型
     * @return
     * @throws Exception 异常
     */
    JsonViewObject setSignalControlStrategy(String code, Integer command, Integer commandType) throws Exception;

    /**
     * 恢复时间表
     *
     * @param code 信号机编号
     * @return
     */
    JsonViewObject recoverScheduleStrategy(String code) throws Exception;

    /**
     * 相位配时下发
     *
     * @return
     * @throws InterruptedException 中断异常
     */
    JsonViewObject phaseTimingSendStrategy(PhaseTimingSendVO phaseTimingSendVO) throws Exception;

    /**
     * 相位差下发
     */
    JsonViewObject phaseDiffSendStrategy();

    /**
     * 删除信号机配置参数
     * 目前支持相位、方案、计划、日期（海康）
     *
     * @param delBaseConfigVO
     * @return
     * @throws Exception
     */
    JsonViewObject delBaseConfig(DelBaseConfigVO delBaseConfigVO) throws Exception;

    /**
     * 信号机校时
     *
     * @param crossCode 信号机编号
     * @param date      日期
     * @return
     * @throws Exception 异常
     */
    JsonViewObject timing(String crossCode, Date date) throws Exception;

    /**
     * 临时方案
     * @return
     * @throws Exception
     */
    JsonViewObject tempScheme(SchemeOptSendVO schemeOptSendVO) throws Exception;

    /**
     * 临时方案详情
     * @param tempSchemeSendVO
     * @return
     * @throws Exception
     */
    JsonViewObject tempSchemeDetail (TempSchemeSendVO tempSchemeSendVO) throws Exception;

    /**
     * 保存到UTC
     * @param saveToUtcPO
     * @return
     * @throws Exception
     */
    JsonViewObject saveToUtc(SaveToUtcPO saveToUtcPO) throws Exception;

    /**
     * 延时相位
     * @param extendPhase
     * @return
     * @throws Exception
     */
    JsonViewObject extendPhase (ExtendPhaseDTO extendPhase) throws Exception;

    /**
     * 步进相位
     * @param stepingPhaseDTO
     * @return
     * @throws Exception
     */
    JsonViewObject stepingPhase (StepingPhaseDTO stepingPhaseDTO) throws Exception;
}
